@@ -355,9 +355,7 @@ struct ValueFlowAnalyzer : Analyzer {
             /* Truncate value */
             const ValueType *dst = tok->valueType();
             if (dst) {
-                const size_t sz = ValueFlow::getSizeOf(*dst,
-                                                       settings,
-                                                       ValueFlow::Accuracy::ExactOrZero);
+                const size_t sz = dst->getSizeOf(settings, ValueType::Accuracy::ExactOrZero, ValueType::SizeOf::Pointer);
                 if (sz > 0 && sz < sizeof(MathLib::biguint)) {
                     MathLib::bigint newvalue = ValueFlow::truncateIntValue(value->intvalue, sz, dst->sign);
 
@@ -608,6 +606,8 @@ private:
             for (const ValueFlow::Value& v:ref->astOperand1()->values()) {
                 if (!v.isLocalLifetimeValue())
                     continue;
+                if (v.conditional)
+                    continue;
                 if (lifeTok)
                     return Action::None;
                 lifeTok = v.tokvalue;
@@ -647,7 +647,8 @@ private:
         if (invalid())
             return Action::Invalid;
         // Follow references
-        auto refs = followAllReferences(tok);
+        // TODO: avoid copy
+        auto refs = tok->refs();
         const bool inconclusiveRefs = refs.size() != 1;
         if (std::none_of(refs.cbegin(), refs.cend(), [&](const ReferenceToken& ref) {
             return tok == ref.token;
@@ -838,6 +839,8 @@ static bool bifurcate(const Token* tok, const std::set<nonneg int>& varids, cons
         const Variable* var = tok->variable();
         if (!var)
             return false;
+        if (!var->isLocal() && !var->isArgument())
+            return false;
         const Token* start = var->declEndToken();
         if (!start)
             return false;
@@ -941,8 +944,11 @@ struct MultiValueFlowAnalyzer : ValueFlowAnalyzer {
     MultiValueFlowAnalyzer(const std::unordered_map<const Variable*, ValueFlow::Value>& args, const Settings& set)
         : ValueFlowAnalyzer(set) {
         for (const auto& p:args) {
-            values[p.first->declarationId()] = p.second;
-            vars[p.first->declarationId()] = p.first;
+            const auto declId = p.first->declarationId();
+            if (declId == 0)
+                continue; // TODO: should never happen?
+            values[declId] = p.second;
+            vars[declId] = p.first;
         }
     }
 
@@ -1075,6 +1081,7 @@ struct MultiValueFlowAnalyzer : ValueFlowAnalyzer {
             const Variable* var = vars.at(p.first);
             if (!var)
                 continue;
+            assert(var->nameToken());
             ps[var->nameToken()] = p.second;
         }
         return ps;
@@ -1489,7 +1496,12 @@ struct ContainerExpressionAnalyzer : ExpressionAnalyzer {
             const Library::Container::Action action = container->getAction(tok->astParent()->strAt(1));
             if (action == Library::Container::Action::PUSH || action == Library::Container::Action::POP || action == Library::Container::Action::APPEND) { // TODO: handle more actions?
                 std::vector<const Token*> args = getArguments(tok->tokAt(3));
-                if (args.size() < 2 || action == Library::Container::Action::APPEND)
+                bool isVariadic = false;
+                if (const Library::Function* libFunc = settings.library.getFunction(tok->tokAt(2))) {
+                    const auto& argChecks = libFunc->argumentChecks;
+                    isVariadic = argChecks.find(-1) != argChecks.end() && argChecks.at(-1).variadic;
+                }
+                if (args.size() < 2 || action == Library::Container::Action::APPEND || isVariadic)
                     return Action::Read | Action::Write | Action::Incremental;
             }
         }
